@@ -22,6 +22,7 @@
 #include <config.h>
 #endif
 
+#include <errno.h>
 #include <time.h>
 #include <sys/types.h>
 #include <string.h>
@@ -33,6 +34,7 @@
 #include "vinagre-notebook.h"
 #include "vinagre-fav.h"
 #include "vinagre-prefs.h"
+#include "vinagre-cache-prefs.h"
 #include "vinagre-utils.h"
 #include "vinagre-bookmarks.h"
 #include "vinagre-ui.h"
@@ -40,6 +42,7 @@
 #include "vinagre-bookmarks-entry.h"
 #include "vinagre-plugin.h"
 #include "vinagre-plugins-engine.h"
+#include "vinagre-dirs.h"
 
 #ifdef VINAGRE_ENABLE_AVAHI
 #include "vinagre-mdns.h"
@@ -56,10 +59,11 @@ vinagre_window_dispose (GObject *object)
 {
   VinagreWindow *window = VINAGRE_WINDOW (object);
 
-  if (window->priv->fav_entry_selected)
+  if (!window->priv->dispose_has_run)
     {
-      g_object_unref (window->priv->fav_entry_selected);
-      window->priv->fav_entry_selected = NULL;
+      vinagre_plugins_engine_deactivate_plugins (vinagre_plugins_engine_get_default (),
+						 window);
+      window->priv->dispose_has_run = TRUE;
     }
 
   if (window->priv->manager)
@@ -99,15 +103,15 @@ vinagre_window_show_hide_controls (VinagreWindow *window)
 {
   if (window->priv->fullscreen)
     {
-      window->priv->fav_panel_visible = GTK_WIDGET_VISIBLE (window->priv->fav_panel);
+      window->priv->fav_panel_visible = gtk_widget_get_visible (window->priv->fav_panel);
       gtk_widget_hide (window->priv->fav_panel);
 
-      window->priv->toolbar_visible = GTK_WIDGET_VISIBLE (window->priv->toolbar);
+      window->priv->toolbar_visible = gtk_widget_get_visible (window->priv->toolbar);
       gtk_widget_hide (window->priv->toolbar);
 
       gtk_widget_hide (window->priv->menubar);
 
-      window->priv->statusbar_visible = GTK_WIDGET_VISIBLE (window->priv->statusbar);
+      window->priv->statusbar_visible = gtk_widget_get_visible (window->priv->statusbar);
       gtk_widget_hide (window->priv->statusbar);
 
       gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->priv->notebook), FALSE);
@@ -138,9 +142,7 @@ vinagre_window_state_event_cb (GtkWidget *widget,
   VinagreWindow *window = VINAGRE_WINDOW (widget);
 
   window->priv->window_state = event->new_window_state;
-  g_object_set (vinagre_prefs_get_default (),
-		"window-state", window->priv->window_state,
-		NULL);
+  vinagre_cache_prefs_set_integer ("window", "window-state", window->priv->window_state);
 
   if ((event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) == 0)
     return FALSE;
@@ -164,13 +166,14 @@ vinagre_window_configure_event (GtkWidget         *widget,
 {
   VinagreWindow *window = VINAGRE_WINDOW (widget);
 
-  window->priv->width  = event->width;
-  window->priv->height = event->height;
+  if ((vinagre_cache_prefs_get_integer ("window", "window-state", 0) & GDK_WINDOW_STATE_MAXIMIZED) == 0)
+    {
+      window->priv->width  = event->width;
+      window->priv->height = event->height;
 
-  g_object_set (vinagre_prefs_get_default (),
-		"window-width", window->priv->width,
-		"window-height", window->priv->height,
-		NULL);
+      vinagre_cache_prefs_set_integer ("window", "window-width", window->priv->width);
+      vinagre_cache_prefs_set_integer ("window", "window-height", window->priv->height);
+    }
 
   return GTK_WIDGET_CLASS (vinagre_window_parent_class)->configure_event (widget, event);
 }
@@ -394,8 +397,9 @@ create_menu_bar_and_toolbar (VinagreWindow *window,
   gtk_ui_manager_add_ui_from_file (manager, vinagre_utils_get_ui_xml_filename (), &error);
   if (error != NULL)
     {
-      g_warning (_("Could not merge vinagre-ui.xml: %s"), error->message);
+      g_critical (_("Could not merge UI XML file: %s"), error->message);
       g_error_free (error);
+      return;
     }
 
   /* Bookmarks */
@@ -473,52 +477,7 @@ fav_panel_size_allocate (GtkWidget     *widget,
 {
   window->priv->side_panel_size = allocation->width;
   if (window->priv->side_panel_size > 0)
-    g_object_set (vinagre_prefs_get_default (),
-		  "side-panel-size", window->priv->side_panel_size,
-		  NULL);
-}
-
-static void
-fav_panel_activated (VinagreFav            *fav,
-		     VinagreBookmarksEntry *entry,
-		     VinagreWindow         *window)
-{
-  g_return_if_fail (vinagre_bookmarks_entry_get_node (entry) == VINAGRE_BOOKMARKS_ENTRY_NODE_CONN);
-
-  vinagre_cmd_open_bookmark (window, vinagre_bookmarks_entry_get_conn (entry));
-}
-
-static void
-fav_panel_selected (VinagreFav            *fav,
-		    VinagreBookmarksEntry *entry,
-		    VinagreWindow         *window)
-{
-  GtkAction *action1, *action2;
-
-  action1 = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					 "BookmarksDel");
-  action2 = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					 "BookmarksEdit");
-
-  if (window->priv->fav_entry_selected)
-    {
-      g_object_unref (window->priv->fav_entry_selected);
-      window->priv->fav_entry_selected = NULL;
-    }
-
-  if (entry)
-    {
-      window->priv->fav_entry_selected = g_object_ref (entry);
-      gtk_action_set_sensitive (action1, TRUE);
-      gtk_action_set_sensitive (action2, TRUE);
-      /* TODO: Change the menuitem label */
-    }
-  else
-    {
-      /* TODO: Change the menuitem label */
-      gtk_action_set_sensitive (action1, FALSE);
-      gtk_action_set_sensitive (action2, FALSE);
-    }
+    vinagre_cache_prefs_set_integer ("window", "side-panel-size", window->priv->side_panel_size);
 }
 
 static void
@@ -598,8 +557,8 @@ vinagre_window_populate_bookmarks (VinagreWindow *window,
 					 action);
 	    g_signal_connect (action,
 			     "activate",
-			     G_CALLBACK (vinagre_cmd_bookmarks_open),
-			     window);
+			     G_CALLBACK (vinagre_fav_bookmarks_open),
+			     window->priv->fav_panel);
 
 	    path = g_strdup_printf ("/MenuBar/BookmarksMenu/%s%s", group, parent?parent:"");
 	    gtk_ui_manager_add_ui (p->manager,
@@ -638,8 +597,8 @@ vinagre_window_update_bookmarks_list_menu (VinagreWindow *window)
   for (l = actions; l != NULL; l = l->next)
     {
       g_signal_handlers_disconnect_by_func (GTK_ACTION (l->data),
-					    G_CALLBACK (vinagre_cmd_bookmarks_open),
-					    window);
+					    G_CALLBACK (vinagre_fav_bookmarks_open),
+					    window->priv->fav_panel);
       gtk_action_group_remove_action (p->bookmarks_list_action_group,
 				      GTK_ACTION (l->data));
     }
@@ -666,73 +625,47 @@ create_side_panel (VinagreWindow *window)
 		   FALSE, 
 		   FALSE);
 
-  g_object_get (vinagre_prefs_get_default (),
-		"side-panel-size", &(window->priv->side_panel_size),
-		NULL);
+  window->priv->side_panel_size = vinagre_cache_prefs_get_integer ("window", "side-panel-size", 200);
   gtk_paned_set_position (GTK_PANED (window->priv->hpaned), window->priv->side_panel_size);
 
   g_signal_connect (window->priv->fav_panel,
 		    "size-allocate",
 		    G_CALLBACK (fav_panel_size_allocate),
 		    window);
-  g_signal_connect (window->priv->fav_panel,
-		    "fav-activated",
-		    G_CALLBACK (fav_panel_activated),
-		    window);
-  g_signal_connect (window->priv->fav_panel,
-		    "fav-selected",
-		    G_CALLBACK (fav_panel_selected),
-		    window);
 }
 
 static void
 init_widgets_visibility (VinagreWindow *window)
 {
-  GdkWindowState state;
-  gint w, h;
   GtkAction *action;
   gboolean visible;
-  VinagrePrefs *prefs = vinagre_prefs_get_default ();
-
-  /* Remove and Edit bookmarks starts disabled */
-  action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"BookmarksDel");
-  gtk_action_set_sensitive (action, FALSE);
-
-  action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					"BookmarksEdit");
-  gtk_action_set_sensitive (action, FALSE);
 
   /* side panel visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					"ViewSidePanel");
-  g_object_get (prefs, "side-panel-visible", &visible, NULL);
+  visible = vinagre_cache_prefs_get_boolean ("window", "side-panel-visible", TRUE);
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
   /* toolbar visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					"ViewToolbar");
-  g_object_get (prefs, "toolbar-visible", &visible, NULL);
+  visible = vinagre_cache_prefs_get_boolean ("window", "toolbar-visible", TRUE);
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
   /* statusbar visibility */
   action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					"ViewStatusbar");
-  g_object_get (prefs, "statusbar-visible", &visible, NULL);
+  visible = vinagre_cache_prefs_get_boolean ("window", "statusbar-visible", TRUE);
   if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
-  g_object_get (prefs,
-		"window-state", &state,
-		"window-width", &w,
-		"window-height", &h,
-		NULL);
+  gtk_window_set_default_size (GTK_WINDOW (window),
+			       vinagre_cache_prefs_get_integer ("window", "window-width", 650),
+			       vinagre_cache_prefs_get_integer ("window", "window-height", 500));
 
-  gtk_window_set_default_size (GTK_WINDOW (window), w, h);
-
-  if ((state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
+  if ((vinagre_cache_prefs_get_integer ("window", "window-state", 0) & GDK_WINDOW_STATE_MAXIMIZED) != 0)
     gtk_window_maximize (GTK_WINDOW (window));
   else
     gtk_window_unmaximize (GTK_WINDOW (window));
@@ -769,6 +702,49 @@ create_notebook (VinagreWindow *window)
   gtk_widget_show (GTK_WIDGET (window->priv->notebook));
 }
 
+static gboolean
+vinagre_window_check_first_run (VinagreWindow *window)
+{
+  gchar *dir, *filename;
+
+  dir = vinagre_dirs_get_user_data_dir ();
+  filename = g_build_filename (dir,
+			       "first_run",
+			       NULL);
+  g_free (dir);
+
+  if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+    {
+      GError *error = NULL;
+      GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+						  GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
+						  GTK_MESSAGE_INFO,
+						  GTK_BUTTONS_CLOSE,
+						  _("About menu accelerators and keyboard shortcuts"));
+      gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						_("Vinagre comes with menu accelerators and keyboard shortcuts disabled by default. The reason is to avoid the keys to be intercepted by the program, and allow them to be sent to the remote machine.\n\nYou can change this behavior through the preferences dialog. For more information, check the documentation.\n\nThis message will appear only once."));     
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+
+      if (vinagre_utils_create_dir (filename, &error))
+        {
+          int fd = g_creat (filename, 0644);
+          if (fd < 0)
+            g_warning (_("Error while creating the file %s: %s"), filename, strerror (errno));
+          else
+            close (fd);
+        }
+      else
+        {
+          g_warning (_("Error while creating the file %s: %s"), filename, error ? error->message: _("Unknown error"));
+          g_clear_error (&error);
+        }
+    }
+
+  g_free (filename);
+  return FALSE;
+}
+
 static void
 vinagre_window_init (VinagreWindow *window)
 {
@@ -777,8 +753,8 @@ vinagre_window_init (VinagreWindow *window)
   gtk_window_set_default_icon_name ("vinagre");
 
   window->priv = VINAGRE_WINDOW_GET_PRIVATE (window);
-  window->priv->fav_entry_selected = NULL;
   window->priv->fullscreen = FALSE;
+  window->priv->dispose_has_run = FALSE;
 
   main_box = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), main_box);
@@ -822,6 +798,12 @@ vinagre_window_init (VinagreWindow *window)
                             G_CALLBACK (vinagre_window_update_bookmarks_list_menu),
                             window);
 #endif
+
+  vinagre_plugins_engine_activate_plugins (vinagre_plugins_engine_get_default (),
+					   window);
+
+
+  g_idle_add ((GSourceFunc) vinagre_window_check_first_run, window);
 }
 
 VinagreNotebook *
@@ -992,28 +974,38 @@ VinagreTab *
 vinagre_window_conn_exists (VinagreWindow *window, VinagreConnection *conn)
 {
   VinagreConnection *c;
-  VinagreTab *tab = NULL;
-  GList *l;
+  VinagreTab *tab;
+  const gchar *host, *protocol;
+  gint port;
+  GList *conns, *l;
 
   g_return_val_if_fail (VINAGRE_IS_WINDOW (window), NULL);
   g_return_val_if_fail (VINAGRE_IS_CONNECTION (conn), NULL);
 
-  l = vinagre_window_get_connections (window);
+  host = vinagre_connection_get_host (conn);
+  protocol = vinagre_connection_get_protocol (conn);
+  port = vinagre_connection_get_port (conn);
+
+  if (!host || !protocol)
+    return NULL;
+
+  l = conns = vinagre_window_get_connections (window);
+  tab = NULL;
 
   while (l != NULL)
     {
       c = VINAGRE_CONNECTION (l->data);
 
-      if (!strcmp (vinagre_connection_get_host (conn), vinagre_connection_get_host (c)) &&
-	  vinagre_connection_get_port (conn) == vinagre_connection_get_port (c) &&
-	  vinagre_connection_get_protocol (conn) == vinagre_connection_get_protocol (c))
+      if (!strcmp (host, vinagre_connection_get_host (c)) &&
+	  !strcmp (protocol, vinagre_connection_get_protocol (c)) &&
+	  port == vinagre_connection_get_port (c))
 	{
 	  tab = g_object_get_data (G_OBJECT (c), VINAGRE_TAB_KEY);
 	  break;
 	}
       l = l->next;
     }
-  g_list_free (l);
+  g_list_free (conns);
 
   return tab;
 }
